unit uperlin;

// __PTC_RESIZE_WINDOW__        Resizable window
// __PTC_ALLOW_CLOSE__          window can be closed
// __PTC_CLOSE_ON_ESCAPE__      Close window when escape is pressed
// __PTC_WINDOWED__
// __PTC_SYSTEM_MENU__
// __PTC_CENTER_WINDOW__
// __PTC_DISABLE_SCREENSAVER__
// __PTC_ICON__                 Show Icon
// __PTC_DDRAW__
// __PTC_GDI__
// __PTC_VFW__

interface

procedure MainFunc;

implementation

uses
  mmsystem,
  windows,
  messages,
{$ifdef __PTC_VFW__}
  vfw;
{$endif}
{$ifdef __PTC_DDRAW__}
  ddraw;
{$endif}
{$ifdef __PTC_DDRAW7__}
  ddraw7;
{$endif}
{$ifdef __PTC_GDI__}
  gdi;
{$endif}

const
  B:  Integer = $100;

type
  PByteA = ^TByteA;
  TByteA = array [Word] of Byte;
  TQuad = packed record
  case integer of
    0: (Color: Longint);
    1: (b1,b2,b3,b4: byte);
  end;

var
  Width:  Integer;
  Height: Integer;
  Size:   Integer;
  pixel: array of Longint;
  map: array of array of Byte;
  Pal: array[0..255] of TQUAD;
  P:  array[0..$202] of Integer;
  G3i: array[0..$202,0..2] of Integer;
  st2: longword;
  SinTable: array[0..1023] of Integer;

function FloatToStr(Value: Extended): string;
begin
  Result := '';
  if Value > 1000 then
    Result := Result + Char(Ord('0') + Trunc(Value/1000));
  if Value > 100 then begin
    Value := value - 1000 * Trunc(Value/1000);
    Result := Result + Char(Ord('0') + Trunc(Value/100));
  end;
  if Value > 10 then begin
    Value := value - 100 * Trunc(Value/100);
    Result := Result + Char(Ord('0') + Trunc(Value/10));
  end;
  Value := value - 10 * Trunc(Value/10);
  Result := Result + Char(Ord('0') + Trunc(Value)) + '.';
  Value := value - Trunc(Value);
  Value := Value * 10;
  Result := Result + Char(Ord('0') + Trunc(Value));
  Value := value - Trunc(Value);
  Value := Value * 10;
  Result := Result + Char(Ord('0') + Trunc(Value));
end;


procedure InitsinTable;
var
  i: integer;
begin
  for i := 0 to 1023 do
    SinTable[i] := round(255 * sin(i/ 1024 * 2 * 3.141529));
end;

procedure InitNoise(seed: Integer);
var
  I,J,T: integer;
  len: double;
  G3: array[0..2] of double;
begin
  if seed <> 0 then
    randseed := seed
  else
    randomize;  

  for i := 0 to B - 1 do begin
    P[i] := i;

		G3[0] := (Trunc(Random * 2 * B) - B)/B;
		G3[1] := (Trunc(Random * 2 * B) - B)/B;
		G3[2] := (Trunc(Random * 2 * B) - B)/B;
    len := sqrt(G3[0] * G3[0] + G3[1] * G3[1] + G3[2] * G3[2]);
    if len > 1E-5 then begin
      G3[0] := G3[0] / len;
      G3[1] := G3[1] / len;
      G3[2] := G3[2] / len;
    end;

		G3i[i,0] := trunc(G3[0] * 1024);
		G3i[i,1] := trunc(G3[1] * 1024);
		G3i[i,2] := trunc(G3[2] * 1024);
  end;

  for i := 0 to B - 1 do begin
    j := Trunc(Random * B);
    T := P[i];
    P[i] := P[j];
    P[j] := T;
  end;

  for i := 0 to B + 1 do begin
    G3i[B + i][0] := G3i[i][0];
    G3i[B + i][1] := G3i[i][1];
    G3i[B + i][2] := G3i[i][2];
  end;
end;

procedure CreatePalette;
var
  i: integer;
begin
  for i := 0 to 255 do begin
    Pal[i].b3 := i;
    Pal[i].b2 := i;
    Pal[i].b1 := round(sqrt(i/255) * 255);
  end;
end;

function Noise3Di(const x,y,z: Integer): Integer;
var
  bx0,bx1,by0,by1,bz0,bz1: integer;
  b00,b10,b01,b11: integer;
  rx0, rx1, ry0, ry1, rz0, rz1: integer;
  sx, sy, sz, t,a,b,c,d,u,v: integer;
  i,j: integer;
  r: integer;
begin
  t := x + $40000000;
  bx0 := (t shr 10) and $FF;
  bx1 := (bx0 + 1) and $FF;
  i := P[bx0];
  rx0 := t and 1023;
  rx1 := rx0 - 1024;

  t := y + $40000000;
  by0 := (t shr 10) and $FF;
  by1 := (by0 + 1) and $FF;
  j := P[bx1];
  ry0 := t and 1023;
  ry1 := ry0 - 1024;

  t := z + $40000000;
  bz0 := (t shr 10) and $FF;
  bz1 := (bz0 + 1) and $FF;
  rz0 := t and 1023;
  rz1 := rz0 - 1024;

  b00 := P[i + by0];
  b10 := P[j + by0];
  b01 := P[i + by1];
  b11 := P[j + by1];

  sx := (rx0 * rx0 * (3072 - 2 * rx0)) shr 20;
  sy := (ry0 * ry0 * (3072 - 2 * ry0)) shr 20;
  sz := (rz0 * rz0 * (3072 - 2 * rz0)) shr 20;

  u := rx0 * G3i[b00 + bz0][0] + ry0 * G3i[b00 + bz0][1] + rz0 * G3i[b00 + bz0][2];
  v := rx1 * G3i[b10 + bz0][0] + ry0 * G3i[b10 + bz0][1] + rz0 * G3i[b10 + bz0][2];
  a := (u shl 10) + sx * (v - u);
asm
  SAR a,20
end;

  u := rx0 * G3i[b01 + bz0][0] + ry1 * G3i[b01 + bz0][1] + rz0 * G3i[b01 + bz0][2];
  v := rx1 * G3i[b11 + bz0][0] + ry1 * G3i[b11 + bz0][1] + rz0 * G3i[b11 + bz0][2];
  b := (u shl 10)  + sx * (v - u);
asm
  SAR b,20
end;

  c := a shl 10 + sy * (b - a);

  u := rx0 * G3i[b00 + bz1][0] + ry0 * G3i[b00 + bz1][1] + rz1 * G3i[b00 + bz1][2];
  v := rx1 * G3i[b10 + bz1][0] + ry0 * G3i[b10 + bz1][1] + rz1 * G3i[b10 + bz1][2];
  a := u shl 10 + sx * (v - u);
asm
  SAR a,20
end;

  u := rx0 * G3i[b01 + bz1][0] + ry1 * G3i[b01 + bz1][1] + rz1 * G3i[b01 + bz1][2];
  v := rx1 * G3i[b11 + bz1][0] + ry1 * G3i[b11 + bz1][1] + rz1 * G3i[b11 + bz1][2];
  b := u shl 10 + sx * (v - u);
asm
  SAR b,20
end;

  d := a shl 10 + sy * (b - a);

  r := c shl 10  + sz * (d - c);
asm
  SAR r,20
end;

  result := r;
end;

function Turbulence3Di(x,y,z: Integer; const n: integer): Integer;
var
  r, i: integer;
  a: integer;
begin
  r := 0;
  a := 1;

  for i := n - 1 downto 0 do begin
    Inc(r, Noise3Di(x,y,z) div a);
    x := x shl 1;
    y := y shl 1;
    z := z shl 1;
    a := a shl 1;
  end;
  Result := abs(r);
end;

procedure BuildPicture;
var
  x,y: longint;
  v: integer;
  z: Integer;
  p,p1,p2: PByteA;
begin
  z := (timeGetTime - st2) div 3;

  for y := 0 to (Height div 2)  do begin
    p := @Map[(y * 2), Width];
    for x := (Width div 2) downto 0 do begin
      v := turbulence3Di(x shl 5, y shl 5, z, 4);
//      v := abs(round(sin(x/44 + v/1024 - 0.4) * cos(y/24 + v/1024 + 1) * 255));
      v := (abs(SinTable[(y * 7 + v div 8 - 120) and 1023] * SinTable[(x shl 2 + v div 8 - 80) and 1023])) shr 8;
      p[0] := v;
      dec(Integer(p),2);
    end;
  end;

  for y := 0 to (Height div 2) do begin
    p := @Map[(y * 2),0];
    for x := (Width div 2) - 1 downto 0 do begin
      p[1] := (p[0] + p[2]) shr 1;
      inc(Integer(p),2);
    end;
  end;

  for y := 0 to (Height div 2) - 1 do begin
    p1 := @Map[(y shl 1),0];
    p  := @Map[(y shl 1) + 1,0];
    p2 := @Map[(y shl 1) + 2,0];
    for x := Width - 1 downto 0 do begin
      p[x] := (p1[x] + p2[x]) shr 1;
    end;
  end;

  for y := 0 to Height - 1 do begin
    for x := Width - 1 downto 0 do begin
      Pixel[x + y * Width] := LongWord(Pal[Map[y,x]]);
    end;
  end;
end;

procedure MainFunc;
var
{$ifdef __PTC_WINDOWED__}
  NrFrame:   Integer;
  t, startTime: double;
  s: string;
{$endif}
begin
{$ifdef __PTC_WINDOWED__}
  startTime := timeGetTime * 0.001;
  NrFrame := 0;
{$endif}
  Width := 320;
  Height:= 200;
  Size  := Height * Width;
  SetLength(pixel, Size);
  SetLength(Map, Height + 1, Width + 1);

  st2 := timeGetTime;

  CreatePalette;
  InitsinTable;
  InitNoise(0);

  if (ptc_open('Noise animation', Width, Height) = 0) then
    Exit;
  while (True) do begin
    BuildPicture;

    ptc_update(@pixel[0]);

{$ifdef __PTC_WINDOWED__}
    Inc(NrFrame);
    t := (timeGetTime * 0.001) - StartTime;
    if t > 2 then begin
      s := 'Noise animation ' + FloatToStr(NrFrame/t) + ' fps';

      SendMessage(wnd, WM_SETTEXT, 0, Longint(Pchar(s)));

      startTime := timeGetTime * 0.001;
      NrFrame := 0;
    end;
{$endif}
  end;
  ptc_close;
end;

end.
